@@ -4,6 +4,20 @@
     *import = \&Exporter::import;
     require DynaLoader;
 
+    use constant _UOK_T         => 1;
+    use constant _IOK_T         => 2;
+    use constant _NOK_T         => 3;
+    use constant _POK_T         => 4;
+    use constant _MATH_MPFR_T   => 5;
+    use constant _MATH_GMPf_T   => 6;
+    use constant _MATH_GMPq_T   => 7;
+    use constant _MATH_GMPz_T   => 8;
+    use constant _MATH_GMP_T    => 9;
+    use constant _MATH_MPC_T    => 10;
+
+use subs qw( __GNU_MP_VERSION __GNU_MP_VERSION_MINOR __GNU_MP_VERSION_PATCHLEVEL
+             __GMP_CC __GMP_CFLAGS);
+
 use overload
     '+'    => \&overload_add,
     '-'    => \&overload_sub,
@@ -45,6 +59,7 @@ use overload
     'abs'  => \&overload_abs;
 
     @Math::GMPz::EXPORT_OK = qw(
+__GNU_MP_VERSION __GNU_MP_VERSION_MINOR __GNU_MP_VERSION_PATCHLEVEL
 Rmpz_abs Rmpz_add Rmpz_add_ui Rmpz_addmul Rmpz_addmul_ui Rmpz_and Rmpz_bin_ui
 Rmpz_bin_uiui Rmpz_cdiv_q Rmpz_cdiv_q_2exp Rmpz_cdiv_q_ui Rmpz_cdiv_qr 
 Rmpz_cdiv_qr_ui Rmpz_cdiv_r Rmpz_cdiv_r_2exp Rmpz_cdiv_r_ui Rmpz_cdiv_ui 
@@ -81,7 +96,7 @@ Rmpz_ui_sub Rmpz_urandomb Rmpz_urandomm Rmpz_xor
 rand_init rand_clear
 TRmpz_out_str TRmpz_inp_str
     );
-    $Math::GMPz::VERSION = '0.26';
+    $Math::GMPz::VERSION = '0.27';
 
     DynaLoader::bootstrap Math::GMPz $Math::GMPz::VERSION;
 
@@ -164,13 +179,13 @@ sub new {
     if(!$type) {die "Inappropriate argument supplied to new()"}
 
     # Create a Math::GMPz object that has $arg1 as its value.
-    # Die if there are any additional args (unless $type == 4)
-    if($type == 1 || $type == 2) { # UOK or IOK
+    # Die if there are any additional args (unless $type == _POK_T)
+    if($type == _UOK_T || $type == _IOK_T) {
       if(@_ ) {die "Too many arguments supplied to new() - expected only one"}
       return Rmpz_init_set_str($arg1, 10);
     }
 
-    if($type == 3) { # NOK
+    if($type == _NOK_T) {
       if(@_ ) {die "Too many arguments supplied to new() - expected only one"}
       if(Math::GMPz::_has_longdouble()) {
         return _Rmpz_init_set_ld($arg1);
@@ -179,37 +194,31 @@ sub new {
 
     }
     
-    if($type == 4) { # POK
+    if($type == _POK_T) { # POK
       if(@_ > 1) {die "Too many arguments supplied to new() - expected no more than two"}
       $base = shift if @_;
       if($base < 0 || $base == 1 || $base > 62) {die "Invalid value for base"}
       return Rmpz_init_set_str($arg1, $base);
     }
 
-    if($type == 8 || $type == 9) { # Math::GMPz or Math::GMP object
+    if($type == _MATH_GMPz_T || $type == _MATH_GMP_T) { # Math::GMPz or Math::GMP object
       if(@_) {die "Too many arguments supplied to new() - expected only one"}
       return Rmpz_init_set($arg1);
     }
 }
 
-#sub Rmpz_out_str {
-#    if(@_ == 2) { return _Rmpz_out_str($_[0], $_[1]) }
-#    elsif(@_ == 3) { return _Rmpz_out_str2($_[0], $_[1], $_[2]) }
-#    else {die "Wrong number of arguments supplied to Rmpz_out_str()"}
-#}
-
 sub Rmpz_out_str {
     if(@_ == 2) {
-       die "Inappropriate 1st arg supplied to Rmpz_out_str" if _itsa($_[0]) != 8;
+       die "Inappropriate 1st arg supplied to Rmpz_out_str" if _itsa($_[0]) != _MATH_GMPz_T && _itsa($_[0]) != _MATH_GMP_T;
        return _Rmpz_out_str($_[0], $_[1]);
     }
     if(@_ == 3) {
-      if(_itsa($_[0]) == 8) {return _Rmpz_out_strS($_[0], $_[1], $_[2])}
-      die "Incorrect args supplied to Rmpz_out_str" if _itsa($_[1]) != 8;
+      if(_itsa($_[0]) == _MATH_GMPz_T || _itsa($_[0]) == _MATH_GMP_T) {return _Rmpz_out_strS($_[0], $_[1], $_[2])}
+      die "Incorrect args supplied to Rmpz_out_str" if _itsa($_[1]) != _MATH_GMPz_T && _itsa($_[1]) != _MATH_GMP_T;
       return _Rmpz_out_strP($_[0], $_[1], $_[2]);
     }
     if(@_ == 4) {
-      die "Inappropriate 2nd arg supplied to Rmpz_out_str" if _itsa($_[1]) != 8;
+      die "Inappropriate 2nd arg supplied to Rmpz_out_str" if _itsa($_[1]) != _MATH_GMPz_T && _itsa($_[1]) != _MATH_GMP_T;
       return _Rmpz_out_strPS($_[0], $_[1], $_[2], $_[3]);
     }
     die "Wrong number of arguments supplied to Rmpz_out_str()";
@@ -217,16 +226,16 @@ sub Rmpz_out_str {
 
 sub TRmpz_out_str {
     if(@_ == 3) {
-      die "Inappropriate 3rd arg supplied to TRmpz_out_str" if _itsa($_[2]) != 8;
+      die "Inappropriate 3rd arg supplied to TRmpz_out_str" if _itsa($_[2]) != _MATH_GMPz_T && _itsa($_[2]) != _MATH_GMP_T;
       return _TRmpz_out_str($_[0], $_[1], $_[2]);
     }
     if(@_ == 4) {
-      if(_itsa($_[2]) == 8) {return _TRmpz_out_strS($_[0], $_[1], $_[2], $_[3])}
-      die "Incorrect args supplied to TRmpz_out_str" if _itsa($_[3]) != 8;
+      if(_itsa($_[2]) == _MATH_GMPz_T || _itsa($_[2]) == _MATH_GMP_T) {return _TRmpz_out_strS($_[0], $_[1], $_[2], $_[3])}
+      die "Incorrect args supplied to TRmpz_out_str" if _itsa($_[3]) != _MATH_GMPz_T && _itsa($_[3]) != _MATH_GMP_T;
       return _TRmpz_out_strP($_[0], $_[1], $_[2], $_[3]);
     }
     if(@_ == 5) {
-      die "Inappropriate 4th arg supplied to TRmpz_out_str" if _itsa($_[3]) != 8;
+      die "Inappropriate 4th arg supplied to TRmpz_out_str" if _itsa($_[3]) != _MATH_GMPz_T && _itsa($_[3]) != _MATH_GMP_T;
       return _TRmpz_out_strPS($_[0], $_[1], $_[2], $_[3], $_[4]);
     }
     die "Wrong number of arguments supplied to TRmpz_out_str()";
@@ -440,32 +449,9 @@ sub _rewrite {
 }
 
 sub Rmpz_printf {
-    local $| = 1; # Make sure the output gets presented in the correct sequence.
-    if(@_ == 1) {printf(shift)}
-
-    else {
-      my @fmt = _rewrite(shift);
-      my @args = @_;
-
-     # It's expected that @fmt and @args are of equal size (though in the case
-     # of both perl's and C's printf function that's not always the case).
-     # Each member of @fmt is paired with one and only one member of @args, each
-     # pair of arguments being passed on to either perl's printf function or
-     # the gmp_printf function. Any excess (leftover) arguments are simply ignored.
-      if(@fmt != @args) {warn "Mismatch in number of args provided to Rmpz_printf.",
-                         " Perhaps the function has not parsed the format string as expected"};
-      my $len = @fmt;
-
-
-     # If $fmt[$i] contains a 'Z' or a 'Q' or an 'F', hand over $fmt[$i] and
-     # $args[$i] to the gmp_printf function.
-     # Else let perl's printf function take care of the formatting - though
-     # the gmp_printf function is also probably capable of handling the task.
-      for(my $i = 0; $i < $len; $i++) { 
-         if($fmt[$i] =~ /Z|Q|F/) {wrap_gmp_printf($fmt[$i], $args[$i])}
-         else {printf($fmt[$i], $args[$i])}
-         }
-      }
+    local $| = 1;
+    die "Rmpz_printf must take 2 arguments: format string, and variable" if @_ != 2;
+    wrap_gmp_printf(@_);
 }
 
 sub Rmpz_fprintf {
@@ -477,6 +463,7 @@ sub Rmpz_sprintf {
     die "Rmpz_sprintf must take 3 arguments: buffer, format string, and variable" if @_ != 3;
     my $len = wrap_gmp_sprintf(@_);
     $_[0] = substr($_[0], 0, $len);
+    return $len;
 }
 
 sub Rmpz_sprintf_ret {
@@ -494,6 +481,12 @@ sub query_eratosthenes_string {
     return 0;
 }
 
+sub __GNU_MP_VERSION {return ___GNU_MP_VERSION()}
+sub __GNU_MP_VERSION_MINOR {return ___GNU_MP_VERSION_MINOR()}
+sub __GNU_MP_VERSION_PATCHLEVEL {return ___GNU_MP_VERSION_PATCHLEVEL()}
+sub __GMP_CC {return ___GMP_CC()}
+sub __GMP_CFLAGS {return ___GMP_CFLAGS()}
+
 1;
 
 __END__
@@ -505,7 +498,7 @@ __END__
 =head1 DEPENDENCIES
 
    This module needs the GMP C library - available from:
-   http://swox.com/gmp
+   http://gmplib.org
 
 =head1 DESCRIPTION
 
@@ -513,7 +506,7 @@ __END__
    Basically this module simply wraps nearly all of
    the integer functions provided by that library.
    The documentation below extensively plagiarises
-   the documentation at http://swox.com/gmp/manual.
+   the documentation at http://gmplib.org.
    See the Math::GMPz test suite for examples of
    usage.
 
@@ -577,7 +570,7 @@ __END__
 
 =head1 FUNCTIONS
 
-   See the GMP documentation at http://swox.com/gmp/manual
+   See the GMP documentation at http://gmplib.org.
 
    These next 3 functions are demonstrated above:
    $rop   = Rmpz_init_set_str($str, $base); # 1 < $base < 37
@@ -632,7 +625,6 @@ __END__
    #####################
 
    INITIALIZING INTEGERS
-   http://swox.com/gmp/manual/Initializing-Integers.html
 
    Normally, a variable should be initialized once only or at least be
    cleared, using `Rmpz_clear', between initializations.
@@ -663,7 +655,7 @@ __END__
    ##################
 
    ASSIGNING INTEGERS
-   http://swox.com/gmp/manual/Assigning-Integers.html 
+
 
    Rmpz_set($rop, $op); 
     Assign the value in $op to $rop.
@@ -728,7 +720,7 @@ __END__
    ###################
 
    CONVERTING INTEGERS
-   http://swox.com/gmp/manual/Converting-Integers.html 
+
 
    $ui = Rmpz_get_ui($op);
     Return the value of $op as an `unsigned long'.
@@ -763,7 +755,6 @@ __END__
    ##################
 
    INTEGER ARITHMETIC
-   http://swox.com/gmp/manual/Integer-Arithmetic.html
 
    Rmpz_add($rop, $op1, $op2); 
    Rmpz_add_ui($rop, $op, $ui); 
@@ -800,7 +791,6 @@ __END__
    ################
 
    INTEGER DIVISION
-   http://swox.com/gmp/manual/Integer-Division.html
 
    `cdiv' rounds quotient up towards +infinity, and remainder
           will have the opposite sign to divisor.
@@ -896,7 +886,6 @@ __END__
    ######################
 
    INTEGER EXPONENTIATION
-   http://swox.com/gmp/manual/Integer-Exponentiation.html
 
    Rmpz_powm($rop, $op1, $op2, $op3);
     $rop = ($op1 ** $op2 ) % $op3 
@@ -913,7 +902,6 @@ __END__
    #############
 
    INTEGER ROOTS
-   http://swox.com/gmp/manual/Integer-Roots.html
 
    Rmpz_root($rop, $op, $ui);
     $rop = $op ** (1 / $ui).
@@ -936,7 +924,6 @@ __END__
    ##########################
 
    NUMBER THEORETIC FUNCTIONS
-   http://swox.com/gmp/manual/Number-Theoretic-Functions.html
 
    $si = Rmpz_probab_prime_p($rop, $ui); 
     Determine whether $rop is prime. Return 2 if $rop is
@@ -1049,7 +1036,6 @@ __END__
    ###################
 
    INTEGER COMPARISONS
-   http://swox.com/gmp/manual/Integer-Comparisons.html
 
    $si = Rmpz_cmp($op1, $op2); 
    $si = Rmpz_cmp_d($op, $double); 
@@ -1073,7 +1059,6 @@ __END__
    ##############################
 
    INTEGER LOGIC AND BIT FIDDLING
-   http://swox.com/gmp/manual/Integer-Logic-and-Bit-Fiddling.html
 
    Rmpz_and($rop, $op1, $op2);
     Set $rop to $op1 logical-and $op2.
@@ -1123,7 +1108,6 @@ __END__
    ###############
 
    I/O of INTEGERS
-   http://swox.com/gmp/manual/I-O-of-Integers.html
 
    $bytes_written = Rmpz_out_str([$prefix,] $op, $base [, $suffix]);
     BEST TO USE TRmpz_out_str INSTEAD.
@@ -1209,7 +1193,6 @@ __END__
    #########################
 
    INTEGER IMPORT AND EXPORT
-   http://swox.com/gmp/manual/Integer-Import-and-Export.html
 
    Rmpz_import($rop, $len, $order, $size, $endian, $nails, $bstr);
 
@@ -1234,7 +1217,6 @@ __END__
    ###############################
 
    MISCELLANEOUS INTEGER FUNCTIONS
-   http://swox.com/gmp/manual/Miscellaneous-Integer-Functions.html
 
    $bool = Rmpz_fits_ulong_p($op); 
    $bool = Rmpz_fits_slong_p($op); 
@@ -1333,140 +1315,86 @@ __END__
    OTHER
 
    $GMP_version = Math::GMPz::gmp_v;
-    Returns the version of the GMP library. The function is
-    not exportable. 
+    Returns the version of the GMP library (eg 4.1.3). The function
+    is not exportable. 
+
+   $GMP_cc = Math::GMPz::__GMP_CC;
+   $GMP_cflags = Math::GMPz::__GMP_CFLAGS;
+    Returns respectively the CC and CFLAGS settings that were used
+    to compile the gmp library. (Not exportable.)
+
+   $major = Math::GMPz::__GNU_MP_VERSION;
+   $minor = Math::GMPz::__GNU_MP_VERSION_MINOR;
+   $patchlevel = Math::GMPz::__GNU_MP_VERSION_PATCHLEVEL;
+    Returns respectively the major, minor, and patchlevel numbers
+    for the GMP library version used by Math::GMPz. (These 
+    functions are in @EXPORT_OK and are therefore exportable by
+    request.) 
 
    ################
 
    FORMATTED OUTPUT
 
-   Rmpz_printf($format_string, @variables);
+   NOTE: The format specification can be found at:
+   http://gmplib.org/manual/Formatted-Output-Strings.html#Formatted-Output-Strings
+   However, the use of '*' to take an extra variable for width and
+   precision is not allowed in this implementation. Instead, it is
+   necessary to interpolate the variable into the format string - ie,
+   instead of:
+     Rmpz_printf("%*Zd\n", $width, $mpz);
+   we need:
+     Rmpz_printf("%${width}Zd\n", $mpz);
+ 
+   $si = Rmpz_printf($format_string, $var);
 
-    'Rmpz_printf' accepts format strings similar to the standard C
-    'printf' (and hence also perl's printf function).  A format
-     specification is of the form:
+    This function changed with the release of Math-GMPz-0.27.
+    Now (unlike the GMP counterpart), it is limited to taking 2
+    arguments - the format string, and the variable to be formatted.
+    That is, you can format only one variable at a time.
+    Returns the number of characters written, or -1 if an error
+    occurred.  
 
-      % [flags] [width] [.[precision]] [type] conv
-
-    GMP adds types 'Z', 'Q' and 'F' for Math::GMPz objects,
-    Math::GMPq objects and Math::GMPf objects respectively.
-    'Z', and 'Q' behave like integers.  'Q' will print a '/' and a
-    denominator, if needed.  'F' behaves like a float.  For example:
-
-     Rmpz_printf ("%s is a Math::GMPz object %Zd\n", "here", $z);
-     Rmpz_printf ("a hex rational: %#40Qx\n", $q);
-     Rmpz_printf ("fixed point mpf %.5Ff with 5 decimal places\n", $f);
-
-    The flags accepted are as follows:
-
-     0         pad with zeros (rather than spaces)
-     #         show the base with '0x', '0X' or '0'
-     +         always show a sign
-     (space)   show a space or a '-' sign
-
-    The optional width and precision can be given as a number within
-    the format string, or as an interpolated perl variable - but note
-    that formatting with '*' (for width and precision fields)
-    WON'T currently work.ie the following is not currently supported:
-
-     $places = 5;
-     Rmpz_printf("mpf %.*Ff\n", $places, $f);
-
-    Instead you would need to rewrite this as:
-
-     $places = 5;
-     Rmpz_printf("mpf %.${places}Ff\n", $f);
-
-    The conversions accepted are as follows.  
-
-     a A       hex floats, C99 style
-     c         character
-     d         decimal integer
-     e E       scientific format float
-     f         fixed point float
-     i         same as d
-     g G       fixed or scientific float
-     o         octal integer
-     s         string
-     u         unsigned integer
-     x X       hex integer
-
-    'a' and 'A' are always supported for GMP objects but don't work with
-    perl's printf function. Always call them prefixed with either 'Z',
-    'F' or 'Q' (whichever is appropriate).
-
-    'p' works with the GMP library and with perl (returns the address of
-    the variable), but can segfault if it's used in the Rmpz_printf 
-    function. For this reason I've excluded it from the list above,
-    though you can certainly use it with perl's printf function - even
-    if the perl variable is a gmp object.
-
-    'o', 'x' and 'X' are unsigned for the standard C types, but for
-    types 'Z', 'Q' and 'N' they are signed.  'u' is not meaningful
-    for 'Z', 'Q' and 'N'.
-
-    In the GMP C library, 'n' can be used with any type, even the GMP
-    types - but that functionality does not currently extend to Perl's
-    GMP objects - so 'n' has been excluded from the above list.
-
-    The precision field has it's usual meaning for integer 'Z' and float
-    'F' types, but is currently undefined for 'Q' and should not be used
-    with that.
-
-    Conversions of Math::GMPf objects only ever generate as many 
-    digits as can be accurately represented by the operand, the same as
-    'Rmpf_get_str' does. Zeros will be used if necessary to pad to the 
-    requested precision.  This happens even for an 'f' conversion of a 
-    Math::GMPf object which is an integer, for instance 2^1024 in a 
-    Math::GMPq object of 128 bits precision will only produce about
-    40 digits, then pad with zeros to the decimal point.  An empty 
-    precision field like '%.Fe' or '%.Ff' can be used to specifically
-    request just the significant digits.
-
-    The format string is interpreted as plain ASCII - multibyte
-    characters are not recognised.
-
-    Also, in Rmpz_printf, there's no support for POSIX '$' style 
-    numbered arguments.
-
-   Rmpz_fprintf($fh, $format_string, $var);
+   $si = Rmpz_fprintf($fh, $format_string, $var);
 
     This function (unlike the GMP counterpart) is limited to taking
     3 arguments - the filehandle, the format string, and the variable
     to be formatted. That is, you can format only one variable at a time.
-    Other than that, the rules outlined above wrt Rmpz_printf apply.
+    Returns the number of characters written, or -1 if an error
+    occurred.
 
-   Rmpz_sprintf($buffer, $format_string, $var);
+   $si = Rmpz_sprintf($buffer, $format_string, $var);
 
     This function (unlike the GMP counterpart) is limited to taking
-    3 arguments - the filehandle, the format string, and the variable
+    3 arguments - the buffer, the format string, and the variable
     to be formatted. $buffer must be large enough to accommodate the
     formatted string, and is truncated to the length of that formatted
     string. If you prefer to have the resultant string returned (rather
     than stored in $buffer), use Rmpz_sprintf_ret instead - which will
-    also leave the length of $buffer unaltered. See Rmpz_printf
-    documentation for further info.
+    also leave the length of $buffer unaltered.
+    Returns the number of characters written, or -1 if an error
+    occurred.
 
    $string = Rmpz_sprintf_ret($buffer, $format_string, $var);
 
-    As for Rmpz_sprintf, but returns the formatted string, rather than
+    As for Rmpz_sprintf, but returns the formatted string, as well as
     storing it in $buffer. $buffer needs to be large enough to 
-    accommodate the formatted string. The length of $buffer (but not the
-    contents) will be unaltered.
+    accommodate the formatted string. The length of $buffer will be
+    unaltered.
 
    ###################
     
 =head1 BUGS
 
-   You can get segfaults if you pass the wrong type of
-   argument to the functions - so if you get a segfault, the
-   first thing to do is to check that the argument types 
-   you have supplied are appropriate.
+    You can get segfaults if you pass the wrong type of
+    argument to the functions - so if you get a segfault, the
+    first thing to do is to check that the argument types 
+    you have supplied are appropriate.
 
 =head1 LICENSE
 
-    This perl code is free software; you may redistribute it
-    and/or modify it under the same terms as Perl itself.
+    This program is free software; you may redistribute it and/or 
+    modify it under the same terms as Perl itself.
+    Copyright 2006-2008, Sisyphus
 
 =head1 AUTHOR
 
